<?php

/**
 * Compute a microid for the given identity and service URIs.
 *
 * @param string  $identity  identity URI
 * @param string  @service   service URI
 * @param string  @alorithm  algorithm to use in calculating the hash (default: sha1)
 * @param boolean @legacy    if true, uritypes and algorithm will not be prefixed 
 *   to the generated microid (default: false)
 */
function microid($identity, $service, $algorithm = 'sha1', $legacy = false) {
	$microid = "";

	// add uritypes and algorithm if not using legacy mode
	if (!$legacy) {
		$microid .= substr($identity, 0, strpos($identity, ':')) . "+" .
			substr($service, 0, strpos($service, ':')) . ":" .
			strtolower($algorithm) . ":";
	}

	// try message digest engine
	if (function_exists('hash')) {
		if (in_array(strtolower($algorithm), hash_algos())) {
			return $microid .= hash($algorithm, hash($algorithm, $identity) . hash($algorithm, $service));
		}
	}

	// try mhash engine
	if (function_exists('mhash')) {
		$hash_method = @constant('MHASH_' . strtoupper($algorithm));
		if ($hash_method != null) {
			$identity_hash = bin2hex(mhash($hash_method, $identity));
			$service_hash = bin2hex(mhash($hash_method, $service));
			return $microid .= bin2hex(mhash($hash_method, $identity_hash . $service_hash));
		}
	}

	// direct string function
	if (function_exists($algorithm)) { 
		return $microid .= $algorithm($algorithm($identity) . $algorithm($service));
	}

	exit("unable to find adequate function for algorithm '$algorithm'");
}

?>
